import os
import sys

import matplotlib.pyplot as pl
import numpy as np

# sys.path.prepend(os.path.abspath(os.path.join(__file__, "..", "..", "..")))
# import emcee


def plot_acor(acorfn):
    pass
